<?php
namespace Faturamento\Logic\ApuracaoFaturamento;
/**
 * Created by JetBrains PhpStorm.
 * User: daniellepereira
 * Date: 08/11/12
 * Time: 17:51
 * To change this template use File | Settings | File Templates.
 */

use \Geral\Entities\Servclie;
use \Geral\Entities\Paramgeral;
use \Geral\Repositories\Clienfilial;
use \Geral\Repositories\Serienf;
use \Geral\Repositories\Paramfilial;
use \Geral\Repositories\Novocodigo;
use \Faturamento\Entities\Ctrlapcdfr;
use \Geral\Entities\Ctrlapurfat;
use \Fiscal\Entities\Seqnumeloca;
use \Fiscal\Entities\Localemissao;
use \Fiscal\Entities\Notafiscal;
use \Indeterminado\Entities\Apurfatntabcorr;
use \Geral\Util\DateUtil;

use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;

class ApuracaoGestaoDireta
{
    protected $paramsProcedure = array();

    /**
     * Validacoes a serem feitas antes de liberar a apuracao
     *
     * @static
     * @param integer $nrorg                Numero da organizacao
     * @param integer $cdfilial             Codigo de filial
     * @param integer $cdcliente            Codigo do cliente
     * @param integer $dtapuinicial         Data inicial de apuracao
     * @param integer $dtapurfinal          Data final de apuracao
     * @param string  $idperiodofat         Identficador periodo de faturamento
     * @param integer $dtsaida              Data de saida
     * @param string  $idcorrecaoefetivos   Identificador de correcao de efetivos
     * @param $continua
     * @return string Msg de confirmacao
     */
    public function validaApuracao(
            $nrorg, $cdfilial, $cdcliente, $dtapuinicial, $dtapurfinal, $idperiodofat, $dtsaida, $idcorrecaoefetivos, $continua)
    {
        if ($continua == 'NAO') {
            if (!$dtapuinicial) {
                \Faturamento\Logic\ApuracaoFaturamento\Exception\DataApuracao::informaDataInicioApuracao();
            }
            if (!$dtapurfinal) {
                \Faturamento\Logic\ApuracaoFaturamento\Exception\DataApuracao::informaDataFimApuracao();
            }
            $dtapuinicial = new \TkDateTime($dtapuinicial, \TkDateTime::FORMAT_DATE);
            $dtapurfinal  = new \TkDateTime($dtapurfinal, \TkDateTime::FORMAT_DATE);
            if (($dtapurfinal->isEarlier($dtapuinicial))) {
                \Faturamento\Logic\ApuracaoFaturamento\Exception\DataApuracao::dataFinalMenorDataInicialApuracao();
            }
            if ($dtsaida) {
                $dtsaida = new \TkDateTime($dtsaida, \TkDateTime::FORMAT_DATE);
                if ($dtsaida->isEarlier($dtapurfinal)) {
                    \Faturamento\Logic\ApuracaoFaturamento\Exception\DataApuracao::dataSaidaMenorDataFimApuracao();
                }
            }
            //@todo usar funcao tk_confirm nos casos do switch
            switch ($idperiodofat) {
                case 1:
                    if ($dtapurfinal->intervalToArray($dtapuinicial, \TkDateTime::INTERVAL_DAYS) != 29){
                        \Faturamento\Logic\ApuracaoFaturamento\Exception\PeriodoApuracao::confirmaApuracaoForaPeriodoMensal();
                    }
                    break;
                case 2:
                    if ($dtapurfinal->intervalToArray($dtapuinicial, \TkDateTime::INTERVAL_DAYS) != 14){
                        \Faturamento\Logic\ApuracaoFaturamento\Exception\PeriodoApuracao::confirmaApuracaoForaPeriodoQuinzenal();
                    }
                    break;
                case 3:
                    if ($dtapurfinal->intervalToArray($dtapuinicial, \TkDateTime::INTERVAL_DAYS) != 9){
                        \Faturamento\Logic\ApuracaoFaturamento\Exception\PeriodoApuracao::confirmaApuracaoForaPeriodoDecendial();
                    }
                    break;
                case 4:
                    if ($dtapurfinal->intervalToArray($dtapuinicial, \TkDateTime::INTERVAL_DAYS) != 6){
                        \Faturamento\Logic\ApuracaoFaturamento\Exception\PeriodoApuracao::confirmaApuracaoForaPeriodoSemanal();
                    }
                    break;
            }
        }
        if ($continua == 'OK' || $continua == 'NAO') {
            if ($idcorrecaoefetivos == 'U' || Paramfilial::consultaIdentificadorCorrecaoEfetivos($nrorg, $cdfilial) == 2) {
                /** @var $servClie Servclie */
                $servClie = Servclie::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND CDCLIENTE = ? AND IDATIVATABCOR = ?",
                                                        array($nrorg, $cdfilial, $cdcliente, 'N'));
                if (is_object($servClie)){
                    $nmcliente = $servClie->getCliente()->getCliente()->getNmrazsocclie();
                    \Faturamento\Logic\ApuracaoFaturamento\Exception\TabelaCorrecaoInativa::confirmaTabelaCorrecaoInativa($nmcliente);
                }
            }
        }

    }


    /**
     * Obtem local de emissao em faturamento por filial.
     * @static
     *
     * @param $nrorg
     * @param $cdfilial
     * @param $cdlocalemissao
     *
     * @return string
     */
    public function obtemLocalEmissao($nrorg, $cdfilial, $cdlocalemissao)
    {
        /** @var $paramGeral \Geral\Entities\Paramgeral */
        $paramGeral = Paramgeral::getObjectByWhere("NRORG = ?", array($nrorg));
        $idApuracaoFilialCentralizada = $paramGeral->getIdutapurfilcent();
        //Caso a apuracao do faturamento seja feita por filial centralizadora, nao deve ser informado local de emissao para a apuracao do faturamento
        if ($idApuracaoFilialCentralizada == 'N') {
            $serieNF = Serienf::SeriePadraoPorFilial($nrorg, $cdfilial);
            if (!empty($serieNF) && $serieNF["IDUTLOCAEMIS"] == 'S'){
                if ($serieNF["IDUTFILILCEM"] == 'S') {
                    /** @var $localEmissao Localemissao */
                    return $cdfilial;
                } else {
                    if (!empty($cdlocalemissao)) {
                        return $cdlocalemissao;
                    } else {
                        \Faturamento\Logic\ApuracaoFaturamento\Exception\AtualizaLocalEmissao::informaLocalEmissao();
                    }
                }
            } else {
                return '';
            }
        }

        return "";
    }


    /**
     * Realiza o calculo da apuracao por cliente
     *
     * @param integer $nrorg          Numero da organizacao
     * @param integer $cdfilial       Codigo da filial
     * @param integer $nrapuracfat    Numero da apuracao
     * @param integer $cdcliente      Codigo do cliente
     * @param integer $dtapurinicial  Data inicial de apuracao
     * @param integer $dtapurfinal    Data final de apuracao
     * @param integer $dtsaida        Data de saida
     * @param integer $cdlocalemissao Codigo do local de emissao
     * @param string  $idperiodofat   Identificador do periodo de faturamento
     * @return string|void            Mensagem de retorno
     */
    public function calculaApuracao($nrorg, $cdfilial, $nrapuracfat, $cdcliente, $dtapurinicial, $dtapurfinal,
                                    $dtsaida, $cdlocalemissao, $idperiodofat)
    {
        try {
            \Setforms2\Shared\Registry::getEntityManager()->beginTransaction();
            /** @var $ctrlApuracFat Ctrlapurfat */
            $ctrlApuracFat = Ctrlapurfat::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND NRAPURACFAT = ? AND CDCLIENTE = ?",
                array($nrorg, $cdfilial, $nrapuracfat, $cdcliente));
            if (is_object($ctrlApuracFat)) {
                $ctrlApuracFat->setDtiniapurac(\Geral\Util\DateUtil::getDataDeString($dtapurinicial));
                $ctrlApuracFat->setDtfimapurac(\Geral\Util\DateUtil::getDataDeString($dtapurfinal));
            } else {
                $ctrlApuracFat = new Ctrlapurfat();
                $ctrlApuracFat->setNrorg($nrorg);
                $ctrlApuracFat->setNrapuracfat(\Geral\Util\Novocodigo::geraCodigo($nrorg, 'CTRLAPURFAT', 20));
                $ctrlApuracFat->setCdcliente($cdcliente);
                $ctrlApuracFat->setCdfilial($cdfilial);
                $ctrlApuracFat->setCdfiliapur($cdfilial);
                $ctrlApuracFat->setDtiniapurac(\Geral\Util\DateUtil::getDataDeString($dtapurinicial));
                $ctrlApuracFat->setDtfimapurac(\Geral\Util\DateUtil::getDataDeString($dtapurfinal));
                $ctrlApuracFat->setCdtipofatuct('001');
            }
            $ctrlApuracFat->persist();
            $this->setParamsProcedureApuracao($nrorg, $cdfilial, $cdcliente, $dtapurinicial, $dtapurfinal, $nrapuracfat, $dtsaida);
            $this->executaProcedureApuracaoGestaoDireta();
            //Caso seja necessario atualizar o local de emissao, deve-se realizar os procedimentos abaixo
            if (!empty($localEmissao)) {
                $ntFiscal = Notafiscal::getObjectsByWhere("NRORG = ? AND CDFILIAL = ? AND NRAPURACFAT = ?", array($nrorg, $cdfilial, $nrapuracfat));
                if (!is_array($ntFiscal)) {
                    \Faturamento\Logic\ApuracaoFaturamento\Exception\AtualizaLocalEmissao::NotaFiscalNaoGerada();
                }
                $SerieNF = Serienf::IntervaloLocalEmissao($nrorg, $cdfilial, $cdlocalemissao);
                if (empty($SerieNF)){
                    \Faturamento\Logic\ApuracaoFaturamento\Exception\AtualizaLocalEmissao::CdfrNaoGeradaIntervaloIndeterminado();
                }
                $cdcontador = 'CDFRLOCEMIS'.$SerieNF["CDCTRLNUMENF"].$SerieNF["NRCDFRINI"];
                NovoCodigo::insereNovoCodigoLocalEmissao($SerieNF["CDCTRLNUMENF"], $cdlocalemissao, $cdcontador);
                $nrsequencial = Novocodigo::consultaSequencialLocalEmissao($cdcontador);
                $Ctrlapcdfr = Ctrlapcdfr::getObjectByWhere("NRORG = ? AND CDFILIALCENT = ? AND NRAPURACFATU = ? AND NVL(IDSITUCDFR, 'C') = ?",
                    array($nrorg, $cdfilial, $nrapuracfat, 'C'));
                if (!is_object($Ctrlapcdfr)) {
                    $Ctrlapcdfr = Ctrlapcdfr::getObjectByWhere("NRORG = ? AND CDCTRLNUMENF = ? AND CDLOCALEMIS = ? AND DTCTRLAPCDFR = ?",
                        array($nrorg, $SerieNF["CDCTRLNUMENF"], $cdlocalemissao, $dtapurinicial));
                    if (!is_object($Ctrlapcdfr)) {
                        $seqNumeLoca = Seqnumeloca::getObjectByWhere("NRORG = ? AND CDCTRLNUMENF = ? AND CDLOCALEMIS = ? AND (LPAD($nrsequencial+1,20,'0') BETWEEN ? AND ?)",
                            array($nrorg, $SerieNF["CDCTRLNUMENF"], $cdlocalemissao, "LPAD(NRCDFRINI,20,'0')", "LPAD(NRCDFRFIM,20,'0')"));
                        if (is_object($seqNumeLoca)) {
                            NovoCodigo::atualizaNovoCodigoLocalEmissao($cdcontador, $nrsequencial+1);
                            $nrsequencial = Novocodigo::consultaSequencialLocalEmissao($cdcontador);
                            $Ctrlapcdfr = new Ctrlapcdfr();
                            /** @var $localEmissao Localemissao */
                            $localEmissao = Localemissao::getObjectByWhere("NRORG = ? AND CDFILIAL = ?", array($nrorg, $cdfilial));
                            $ctrlApuracFat = Ctrlapurfat::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND NRAPURACFAT = ?", array($nrorg, $cdfilial, $nrapuracfat));
                            $Ctrlapcdfr->setCtrlnumenf($SerieNF["CDCTRLNUMENF"]);
                            $Ctrlapcdfr->setLocalemis($localEmissao);
                            $Ctrlapcdfr->setCdnrcdfr($nrsequencial);
                            $Ctrlapcdfr->setApuracfatu($ctrlApuracFat);
                            $Ctrlapcdfr->setIdperiodfatu($idperiodofat);
                            $Ctrlapcdfr->setDtctrlapcdfr(\Geral\Util\DateUtil::getDataDeString($dtapurinicial));
                        } else {
                            \Faturamento\Logic\ApuracaoFaturamento\Exception\AtualizaLocalEmissao::CdfrNaoGeradaNumeracaoIndisponivel();
                        }
                    }
                }
            }
            \Setforms2\Shared\Registry::getEntityManager()->flush();
            \Setforms2\Shared\Registry::getEntityManager()->commit();
            return true;
        } catch (\Exception $e) {
            \Setforms2\Shared\Registry::getEntityManager()->rollback();
            throw $e;
        }
    }


    /**
     * Seta os parametros da procedure APURAC_GESDIRETA
     *
     * @param integer $nrorg       Numero da organizacao
     * @param integer $cdfilial    Codigo da filial
     * @param integer $cdcliente   Codigo do cliente
     * @param integer $dtiniapur   Data inicial de apuracao
     * @param integer $dtfinalapur Data final de apuracao
     * @param integer $nrapuracfat Numero de apuracao
     * @param integer $dtsaidaapur Data de saida apuracao
     */
    public function setParamsProcedureApuracao($nrorg, $cdfilial, $cdcliente, $dtiniapur, $dtfinalapur, $nrapuracfat, $dtsaidaapur)
    {
        if ($dtsaidaapur == null) $dtsaidaapur = $dtfinalapur;
        $this->paramsProcedure = array(
//            'P_NRORG'       => $nrorg,
            'P_CDFILIAL'    => $cdfilial,
            'P_CDCLIENTE'   => $cdcliente,
            'P_DATAINI'     => $dtiniapur,
            'P_DATAFIM'     => $dtfinalapur,
            'P_NRAPURACFAT' => $nrapuracfat,
            'P_DTSAIDAPUR'  => $dtsaidaapur,
        );
    }


    /**
     * Executa a procedure APURAC_GESDIRETA
     *
     * @throws Exception\ProcedureApuracao
     * @throws \Exception
     */
    public function executaProcedureApuracaoGestaoDireta() {
        try {
            $procedure = new Procedure("APURAC_GESDIRETA");
            foreach ($this->paramsProcedure as $key => $value) {
                $procedure->addParam(
                    new Param(
                        $key,
                        Param::PARAM_INPUT,
                        $value,
                        Param::PARAM_TYPE_STR
                    )
                );
            }

            $procedure->call();
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Emite o Relatorio “Apuracao x Tabela de Correcao" que mostrará o comparativo da apuracao e a tabela de correcao de servicos.
     * caso a consulta retorne algum valor
     *
     * @param integer $nrorg       Numero da organizacao
     * @param integer $cdfiliapur  Filial de apuracao
     * @param integer $nrapuracfat Numero da apuracao
     * @return bool
     */
    public function exibeRelatorioComparaValoresFaturadoCorrigido($nrorg, $cdfiliapur, $nrapuracfat)
    {
        $Apurfatntabcorr = Apurfatntabcorr::getObjectByWhere("NRORG = ? AND CDFILIAL = ? AND NRAPURACFAT = ?", array($nrorg, $cdfiliapur, $nrapuracfat));
        if (is_object($Apurfatntabcorr)) {
            return true;
        }

        return false;
    }

}